// Copyright (c) 2005 David Drysdale.  See license.txt for license info
#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include <TxtGlue.h>
#include "resourcedefines.h"
#include "constants.h"
#include "boardgadget.h"
#include "mainform.h"
#include "utils.h"
#include "horizscroll.h"
#include "vertscroll.h"
#include "globals.h"

static void UpdateHelpScrollbar(FormPtr form)
{
  UInt16 curPos;
  UInt16 textHeight;
  UInt16 fieldHeight;

  // Set up the scrollbar size and position
  FldGetScrollValues((FieldPtr) GetObjectPtr(form, HelpTextField),
                     &curPos, &textHeight, &fieldHeight);
  SclSetScrollBar((ScrollBarPtr) GetObjectPtr(form, HelpTextScrollbar),
                  curPos, 0, (textHeight - fieldHeight), (fieldHeight - 1));
}

Boolean HelpEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
  MemHandle h;
  FieldPtr field;
  Char *newVal;
  Int16 linesToScroll;
  WChar c;

  form = FrmGetActiveForm();

  switch (event->eType) 
  {
    case frmOpenEvent:
    case frmUpdateEvent:
      // Get at the help text string in a resource
      h = DmGet1Resource('tSTR', MineSweeperHelpString);
      newVal = (Char *) MemHandleLock(h);
      field = (FieldPtr)GetObjectPtr(form, HelpTextField);
      SetFieldTextFromStr(field, newVal, false);
      MemHandleUnlock(h);
      DmReleaseResource(h);
      FldSetScrollPosition(field, 0);
      UpdateHelpScrollbar(form);
      FrmDrawForm(form);
      handled = true;
      break;
 
    case sclRepeatEvent:
      field = (FieldPtr)GetObjectPtr(form, HelpTextField);
      linesToScroll = (event->data.sclRepeat.newValue - event->data.sclRepeat.value);
      if (linesToScroll < 0) {
        FldScrollField(field, (UInt16) -linesToScroll, winUp);
      } else {
        FldScrollField(field, (UInt16) linesToScroll, winDown);
      }
      break;

    case keyDownEvent:
      field = (FieldPtr)GetObjectPtr(form, HelpTextField);
      linesToScroll = FldGetVisibleLines(field) - 1;
      c = event->data.keyDown.chr;
      // Check for a virtual char
      if (TxtGlueCharIsVirtual(event->data.keyDown.modifiers, c)) {
        if ((c == vchrPageUp) && (FldScrollable(field, winUp))) {
          FldScrollField(field, (UInt16)linesToScroll, winUp);
          UpdateHelpScrollbar(form);
          handled = true;
        } else if ((c == vchrPageDown) && (FldScrollable(field, winDown))) {
          FldScrollField(field, (UInt16)linesToScroll, winDown);
          UpdateHelpScrollbar(form);
          handled = true;
        }
      }
      break;

    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID) {
        case HelpOKButton:
          // Return to the main form
          FrmGotoForm(MainForm);
          break;
      }
      break;
      
    default:
      break;
  }  
  return handled;
}

Boolean PreferencesHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
  FieldPtr field;
  Char scratchStr[64];
  Int16 width =32, height=32;
  Int16 newNumMines;

  form = FrmGetActiveForm();

  switch (event->eType) 
  {
    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID) {
        case SmallSizeButton:
        case MediumSizeButton:
        case LargeSizeButton:
          if (event->data.ctlSelect.controlID == SmallSizeButton) {
            width = kSmallWidth;
            height = kSmallHeight;
          } else if (event->data.ctlSelect.controlID == MediumSizeButton) {
            width = kMediumWidth;
            height = kMediumHeight;
          } else /* if (event->data.cltSelect.controlID == LargeSizeButton) */ {
            width = kLargeWidth;
            height = kLargeHeight;
          }
          // Adjust the displayed width, height and number of mines and
          // make sure the width and height fields are not editable
          StrPrintF(scratchStr, "%d", width);
          field = (FieldPtr)GetObjectPtr(form, BoardWidthField);
          SetFieldEditable(field, false);
          SetFieldTextFromStr(field, scratchStr, true);

          StrPrintF(scratchStr, "%d", height);
          field = (FieldPtr)GetObjectPtr(form, BoardHeightField);
          SetFieldEditable(field, false);
          SetFieldTextFromStr(field, scratchStr, true);

          // Update the number of mines depending on the new size
          newNumMines = -1;
          if (FrmGetControlValue(form, FrmGetObjectIndex(form, EasyLevelButton))) {
            if ((width == kSmallWidth) && (height == kSmallHeight)) {
              newNumMines = kSmallEasy;
            } else if ((width == kMediumWidth) && (height == kMediumHeight)) {
              newNumMines = kMediumEasy;
            } else /* if ((width == kLargeWidth) && (height == kLargeHeight)) */ {
              newNumMines = kLargeEasy;
            }
          } else if (FrmGetControlValue(form, FrmGetObjectIndex(form, MediumLevelButton))) {
            if ((width == kSmallWidth) && (height == kSmallHeight)) {
              newNumMines = kSmallMedium;
            } else if ((width == kMediumWidth) && (height == kMediumHeight)) {
              newNumMines = kMediumMedium;
            } else /* if ((width == kLargeWidth) && (height == kLargeHeight)) */ {
              newNumMines = kLargeMedium;
            }
          } else if (FrmGetControlValue(form, FrmGetObjectIndex(form, HardLevelButton))) {
            if ((width == kSmallWidth) && (height == kSmallHeight)) {
              newNumMines = kSmallHard;
            } else if ((width == kMediumWidth) && (height == kMediumHeight)) {
              newNumMines = kMediumHard;
            } else /* if ((width == kLargeWidth) && (height == kLargeHeight)) */ {
              newNumMines = kLargeHard;
            }
          } 
          if (newNumMines != -1) {
            StrPrintF(scratchStr, "%d", newNumMines);
            field = (FieldPtr)GetObjectPtr(form, NumMinesField);
            SetFieldTextFromStr(field, scratchStr, true);
          }
          // Reenable the Easy/Medium/Hard level buttons 
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, EasyLevelButton)), true);
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, MediumLevelButton)), true);
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, HardLevelButton)), true);
          break;
        case CustomSizeButton:
          // Make the width and height fields editable
          field = (FieldPtr)GetObjectPtr(form, BoardWidthField);
          SetFieldEditable(field, true);
          FldDrawField(field);
          field = (FieldPtr)GetObjectPtr(form, BoardHeightField);
          SetFieldEditable(field, true);
          FldDrawField(field);
          // Immediately jump to custom difficulty and disable the other levels
          FrmSetControlValue(form, FrmGetObjectIndex(form, CustomLevelButton), 1);
          FrmSetControlValue(form, FrmGetObjectIndex(form, EasyLevelButton), 0);
          FrmSetControlValue(form, FrmGetObjectIndex(form, MediumLevelButton), 0);
          FrmSetControlValue(form, FrmGetObjectIndex(form, HardLevelButton), 0);
          field = (FieldPtr)GetObjectPtr(form, NumMinesField);
          SetFieldEditable(field, true);
          FldDrawField(field);
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, EasyLevelButton)), false);
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, MediumLevelButton)), false);
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, HardLevelButton)), false);
          break;
        case EasyLevelButton:
        case MediumLevelButton:
        case HardLevelButton:
          // Find the current status of the size button
          newNumMines = -1;
          if (event->data.ctlSelect.controlID == EasyLevelButton) {
            if (FrmGetControlValue(form, FrmGetObjectIndex(form, SmallSizeButton))) {
              newNumMines = kSmallEasy;
            } else if (FrmGetControlValue(form, FrmGetObjectIndex(form, MediumSizeButton))) {
              newNumMines = kMediumEasy;
            } else /* if (FrmGetControlValue(form, FrmGetObjectIndex(form, LargeSizeButton))) */ {
              newNumMines = kLargeEasy;
            }
          } else if (event->data.ctlSelect.controlID == MediumLevelButton) {
            if (FrmGetControlValue(form, FrmGetObjectIndex(form, SmallSizeButton))) {
              newNumMines = kSmallMedium;
            } else if (FrmGetControlValue(form, FrmGetObjectIndex(form, MediumSizeButton))) {
              newNumMines = kMediumMedium;
            } else /* if (FrmGetControlValue(form, FrmGetObjectIndex(form, LargeSizeButton))) */ {
              newNumMines = kLargeMedium;
            }
          } else /* if (event->data.ctlSelect.controlID == HardLevelButton) */ {
            if (FrmGetControlValue(form, FrmGetObjectIndex(form, SmallSizeButton))) {
              newNumMines = kSmallHard;
            } else if (FrmGetControlValue(form, FrmGetObjectIndex(form, MediumSizeButton))) {
              newNumMines = kMediumHard;
            } else /* if (FrmGetControlValue(form, FrmGetObjectIndex(form, LargeSizeButton))) */ {
              newNumMines = kLargeHard;
            }
          } 
          field = (FieldPtr)GetObjectPtr(form, NumMinesField);
          if (newNumMines != -1) {
            // Adjust the displayed number of mines
            StrPrintF(scratchStr, "%d", newNumMines);
            SetFieldTextFromStr(field, scratchStr, true);
          }
          // Make the number of mines non-editable
          SetFieldEditable(field, false);
          FldDrawField(field);
          break;
        case CustomLevelButton:
          // Make the number of mines field editable
          field = (FieldPtr)GetObjectPtr(form, NumMinesField);
          SetFieldEditable(field, true);
          FldDrawField(field);
          break;
      }
      break;
      
    default:
      break;
  }  
  return handled;
}

static void MainFormInit(FormPtr form)
{
  MemHandle h;

  // warning-- don't do any drawing in this routine.
  // Also, don't call FrmSetFocus from here (it must be called *after*
  // FrmDrawForm)

  // Initialize the gadget data for the horizontal scrollbar
  h = MemHandleNew(sizeof(HORIZSCROLL));
  FrmSetGadgetData(form, FrmGetObjectIndex(form, HorizScroll), h);

  // Update the scroll bars
  UpdateVScrollBar(form, VertScroll, currVPos);
  UpdateHScrollBar(form, HorizScroll, currHPos);

}

static void MainFormDeinit(FormPtr form)
{
  MemHandle h;

  h = (MemHandle) FrmGetGadgetData(form, FrmGetObjectIndex(form, HorizScroll));
  if (h) MemHandleFree(h);
}

static void FieldFill(FormPtr dialog, UInt16 fieldID, UInt16 value)
{
  Char scratchStr[64];
  FieldPtr field;
  StrPrintF(scratchStr, "%3d", value);
  field = (FieldPtr)GetObjectPtr(dialog, fieldID);
  SetFieldTextFromStr(field, scratchStr, false);
}


Boolean MainFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormPtr form;
  FormPtr dialog;
  UInt16 hitButton;
  UInt16 boardGadgetIndex, horizScrollIndex;
  RectangleType boardBounds, horizScrollBounds;
  WChar c;
  UInt32 nowTime;
  Char scratchStr[64];
  FieldPtr field;
  UInt16 difficultyID, sizeID;
  
  form = FrmGetActiveForm();

  switch (event->eType) 
  {
    case frmOpenEvent:
      MainFormInit(form);
      BoardShowNumbers(form);
      FrmDrawForm(form);
      // Draw all of our gadgets
      BoardGadgetDraw(form, BoardGadget);
      HorizScrollDraw(form, HorizScroll);
      handled = true;
      break;

    case frmUpdateEvent:
      FrmDrawForm(form);
      // Draw all of our gadgets
      BoardGadgetDraw(form, BoardGadget);
      HorizScrollDraw(form, HorizScroll);
      handled = true;
      break;

    case menuEvent:
      switch (event->data.menu.itemID) {
        case NewGame:
          // Get rid of the menu bar (if present) before we draw underneath it
          MenuEraseStatus(NULL);
          InitBoard(board);
          CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, DefuseButton)), true);
          BoardGadgetDraw(form, BoardGadget);
          handled = true;
          break;
        case HighScores:
          dialog = FrmInitForm(BestTimesDialog);
          // Set the field values for the best times
          FieldFill(dialog, BestTimeSmallEasy, board->smallEasyTime);
          FieldFill(dialog, BestTimeSmallMedium, board->smallMediumTime);
          FieldFill(dialog, BestTimeSmallHard, board->smallHardTime);
          FieldFill(dialog, BestTimeMediumEasy, board->mediumEasyTime);
          FieldFill(dialog, BestTimeMediumMedium, board->mediumMediumTime);
          FieldFill(dialog, BestTimeMediumHard, board->mediumHardTime);
          FieldFill(dialog, BestTimeLargeEasy, board->largeEasyTime);
          FieldFill(dialog, BestTimeLargeMedium, board->largeMediumTime);
          FieldFill(dialog, BestTimeLargeHard, board->largeHardTime);
          FrmDoDialog(dialog);
          FrmDeleteForm(dialog);
          handled = true;
          break;
        case Preferences:
          dialog = FrmInitForm(PreferencesDialog);
          FrmSetEventHandler(dialog, PreferencesHandleEvent);
          // Set the values from the current preferences
          if ((board->width == kSmallWidth) && (board->height == kSmallHeight)) {
            sizeID = SmallSizeButton;
          } else if ((board->width == kMediumWidth) && (board->height == kMediumHeight)) {
            sizeID = MediumSizeButton;
          } else if ((board->width == kLargeWidth) && (board->height == kLargeHeight)) {
            sizeID = LargeSizeButton;
          } else {
            sizeID = CustomSizeButton;
          }
          FrmSetControlValue(dialog, FrmGetObjectIndex(dialog, sizeID), 1);

          StrPrintF(scratchStr, "%d", board->width);
          field = (FieldPtr)GetObjectPtr(dialog, BoardWidthField);
          SetFieldTextFromStr(field, scratchStr, false);
          SetFieldEditable(field, (sizeID == CustomSizeButton));

          StrPrintF(scratchStr, "%d", board->height);
          field = (FieldPtr)GetObjectPtr(dialog, BoardHeightField);
          SetFieldTextFromStr(field, scratchStr, false);
          SetFieldEditable(field, (sizeID == CustomSizeButton));

          difficultyID = CustomLevelButton;
          switch (sizeID) {
            case SmallSizeButton:
              if (board->totalMines == kSmallEasy) difficultyID = EasyLevelButton;
              else if (board->totalMines == kSmallMedium) difficultyID = MediumLevelButton;
              else if (board->totalMines == kSmallHard) difficultyID = HardLevelButton;
              break;
            case MediumSizeButton:
              if (board->totalMines == kMediumEasy) difficultyID = EasyLevelButton;
              else if (board->totalMines == kMediumMedium) difficultyID = MediumLevelButton;
              else if (board->totalMines == kMediumHard) difficultyID = HardLevelButton;
              break;
            case LargeSizeButton:
              if (board->totalMines == kLargeEasy) difficultyID = EasyLevelButton;
              else if (board->totalMines == kLargeMedium) difficultyID = MediumLevelButton;
              else if (board->totalMines == kLargeHard) difficultyID = HardLevelButton;
              break;
          }
          FrmSetControlValue(dialog, FrmGetObjectIndex(dialog, difficultyID), 1);
          if (sizeID == CustomSizeButton) {
            CtlSetEnabled(FrmGetObjectPtr(dialog, FrmGetObjectIndex(dialog, EasyLevelButton)), false);
            CtlSetEnabled(FrmGetObjectPtr(dialog, FrmGetObjectIndex(dialog, MediumLevelButton)), false);
            CtlSetEnabled(FrmGetObjectPtr(dialog, FrmGetObjectIndex(dialog, HardLevelButton)), false);
          }

          StrPrintF(scratchStr, "%d", board->totalMines);
          field = (FieldPtr)GetObjectPtr(dialog, NumMinesField);
          SetFieldTextFromStr(field, scratchStr, false);
          SetFieldEditable(field, (difficultyID == CustomLevelButton));
          
          FrmSetControlValue(dialog, FrmGetObjectIndex(dialog,DoQuestionMark), board->preferences & DOQUESTIONMARKS);

          hitButton = FrmDoDialog(dialog);
          if (hitButton == PreferencesOKButton) {
            // Retrieve the new values from the dialog
            Int16 val;
            Char *newVal;
            Int16 newWidth = board->width;
            Int16 newHeight = board->height;
            Int16 newNumMines = board->totalMines;
            Boolean doRedraw = false;
            
            if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, SmallSizeButton))) {
              newWidth = kSmallWidth;
              newHeight = kSmallHeight;
            } else if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, MediumSizeButton))) {
              newWidth = kMediumWidth;
              newHeight = kMediumHeight;
            } else if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, LargeSizeButton))) {
              newWidth = kLargeWidth;
              newHeight = kLargeHeight;
            } else {
              // Get custom width/height from the text fields
              field = (FieldPtr)GetObjectPtr(dialog, BoardWidthField);
              newVal = FldGetTextPtr(field);
              if (newVal) {
                val = StrAToI(newVal);
                if (val > kMaxWidth) val = kMaxWidth;
                if (val < kMinWidth) val = kMinWidth;
                newWidth = val;
              }
              field = (FieldPtr)GetObjectPtr(dialog, BoardHeightField);
              newVal = FldGetTextPtr(field);
              if (newVal) {
                val = StrAToI(newVal);
                if (val > kMaxHeight) val = kMaxHeight;
                if (val < kMinHeight) val = kMinHeight;
                newHeight = val;
              }
            } 

            if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, EasyLevelButton))) {
              if ((newWidth == kSmallWidth) && (newHeight == kSmallHeight)) {
                newNumMines = kSmallEasy;
              } else if ((newWidth == kMediumWidth) && (newHeight == kMediumHeight)) {
                newNumMines = kMediumEasy;
              } else if ((newWidth == kLargeWidth) && (newHeight == kLargeHeight)) {
                newNumMines = kLargeEasy;
              } else {
                DEBUG((T, "Should never reach this arm: Easy level, custom size"));
                newNumMines = (newWidth*newHeight*kEasyRatio) / kScalingFactor;
              }
            } else if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, MediumLevelButton))) {
              if ((newWidth == kSmallWidth) && (newHeight == kSmallHeight)) {
                newNumMines = kSmallMedium;
              } else if ((newWidth == kMediumWidth) && (newHeight == kMediumHeight)) {
                newNumMines = kMediumMedium;
              } else if ((newWidth == kLargeWidth) && (newHeight == kLargeHeight)) {
                newNumMines = kLargeMedium;
              } else {
                DEBUG((T, "Should never reach this arm: Medium level, custom size"));
                newNumMines = (newWidth*newHeight*kMediumRatio) / kScalingFactor;
              }
            } else if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, HardLevelButton))) {
              if ((newWidth == kSmallWidth) && (newHeight == kSmallHeight)) {
                newNumMines = kSmallHard;
              } else if ((newWidth == kMediumWidth) && (newHeight == kMediumHeight)) {
                newNumMines = kMediumHard;
              } else if ((newWidth == kLargeWidth) && (newHeight == kLargeHeight)) {
                newNumMines = kLargeHard;
              } else {
                DEBUG((T, "Should never reach this arm: Hard level, custom size"));
                newNumMines = (newWidth*newHeight*kHardRatio) / kScalingFactor;
              }
            } else {
              // Get custom number of mines from the text field
              field = (FieldPtr)GetObjectPtr(dialog, NumMinesField);
              newVal = FldGetTextPtr(field);
              if (newVal) {
                val = StrAToI(newVal);
                if (val > kMaxMines) val = kMaxMines;
                if (val < kMinMines) val = kMinMines;
                newNumMines = val;
              }
            } 

            if (newNumMines >= (newWidth*newHeight/2)) {
              // Watch out for lunatics
              newNumMines = newWidth*newHeight/2;
            }

            // If the size is changed, we need a new board
            if ((newWidth != board->width) || (newHeight != board->height)) {
              BOARD *newBoard;
              RectangleType bounds;
              
              newBoard = CreateBoard(newWidth, newHeight, newNumMines);
              InitBoard(newBoard);
              CopyPreferences(newBoard, board);
              // Swap over to the new board
              MemPtrFree(board);
              board = newBoard;
              currVPos = 0;
              currHPos = 0;
              doRedraw = true;
              // Erase all traces of the old board
              FrmGetObjectBounds(form, FrmGetObjectIndex(form, BoardGadget), &bounds);
              WinEraseRectangle(&bounds, 0);
              CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, DefuseButton)), true);
            }

            // If the number of mines has changed, we need a new game
            if (newNumMines != board->totalMines) {
              board->totalMines = newNumMines;
              InitBoard(board);
              CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, DefuseButton)), true);
              doRedraw = true;
            }
            if (doRedraw) {
              BoardGadgetDraw(form, BoardGadget);
              // Redraw the scroll bars because overall size may have changed
              UpdateVScrollBar(form, VertScroll, currVPos);
              UpdateHScrollBar(form, HorizScroll, currHPos);
            }

            val = FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, DoQuestionMark));
            if (val) {
              board->preferences |= DOQUESTIONMARKS; 
            } else {
              board->preferences &= ~DOQUESTIONMARKS;
            }

            if (FrmGetControlValue(dialog, FrmGetObjectIndex(dialog, ResetBestTimes))) {
              ResetTimes(board);
            }
            
          }
          FrmDeleteForm(dialog);
          handled = true;
          break;
        case HelpMineSweeper:
          // Display the help form
          FrmGotoForm(HelpDialog);
          handled = true;
          break;
        case AboutMineSweeper:
          (void) FrmAlert(AboutAlert);
          handled = true;
          break;
      }
      break;

    case penDownEvent:
      boardGadgetIndex = FrmGetObjectIndex(form, BoardGadget);
      horizScrollIndex = FrmGetObjectIndex(form, HorizScroll);
      FrmGetObjectBounds(form, boardGadgetIndex, &boardBounds);
      FrmGetObjectBounds(form, horizScrollIndex, &horizScrollBounds);
      if (RctPtInRectangle(event->screenX, event->screenY, &boardBounds)) {
        BoardGadgetTap(form, BoardGadget, event);
      } else if (RctPtInRectangle(event->screenX, event->screenY, &horizScrollBounds)) {
        HorizScrollTap(form, HorizScroll, event);
      }
      break;

    case sclRepeatEvent:
      ScrollVLines(form, VertScroll, 
                   event->data.sclRepeat.newValue - event->data.sclRepeat.value, false);
      break;

    case keyDownEvent:
      c = event->data.keyDown.chr;
      // Check for a virtual char
      if (TxtGlueCharIsVirtual(event->data.keyDown.modifiers, c)) {
        if ((c == vchrPageUp) || (c == vchrPageDown)) {
          handled = true;
        }
      }
      break;

    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID) {
        case DefuseButton:
          defusePending = !defusePending;
          FrmSetControlValue(form, FrmGetObjectIndex(form, DefuseButton), defusePending);
          break;
      }
      break;
      
    case frmCloseEvent:
      MainFormDeinit(form);
      handled = false;
      break;

    case winExitEvent:
      if (event->data.winExit.exitWindow == (WinHandle)FrmGetFormPtr(MainForm)) dontDraw = true;
      break;
    
    case winEnterEvent:
      if (event->data.winEnter.enterWindow == (WinHandle)FrmGetFormPtr(MainForm)) dontDraw = false;
      break;


    case nilEvent:
      nowTime = TimGetTicks();
      handled = true;
      break;
      
    default:
      break;
  }  
  return handled;
}
