// Copyright (c) 2005 David Drysdale.  See license.txt for license info
#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "utils.h"
#include "resourcedefines.h"

Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
  UInt32 romVersion;

  // See if we're on in minimum required version of the ROM or later.
  FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  if (romVersion < requiredVersion) {
    UInt16 safeToCallAlertFlags;
    
    safeToCallAlertFlags = 
      sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp;
    if ((launchFlags & (safeToCallAlertFlags)) == safeToCallAlertFlags) {
      FrmAlert (RomIncompatibleAlert);
      
      // Pilot 1.0 will continuously relaunch this app unless we switch to 
      // another safe one.
      if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
        AppLaunchWithCommand(sysFileCDefaultApp, 
                             sysAppLaunchCmdNormalLaunch, NULL);
    }
    
    return (sysErrRomIncompatible);
  }
  
  return 0;
}


void *GetObjectPtr(FormPtr form, UInt16 objectID)
{
  return FrmGetObjectPtr(form, FrmGetObjectIndex(form, objectID));
}

void *GetObjectPtrFromActiveForm(UInt16 objectID)
{
  return GetObjectPtr(FrmGetActiveForm(), objectID);
}

Err SetFieldTextFromStr(FieldPtr field, const Char *s, Boolean redraw)
{
   MemHandle h;
   Err err;

   h = FldGetTextHandle(field);
   if (h) {
     FldSetTextHandle(field, NULL);
     err = MemHandleResize(h, StrLen(s) + 1);
     if (err != errNone) {
       // Error resizing -- restore previous contents
       FldSetTextHandle(field, h);  
       return err;
     }
   } else {
     h = MemHandleNew(StrLen(s) + 1);
     if (!h) return memErrNotEnoughSpace;
   }

   // at this point, we have a handle of the correct size

   // copy the string to the locked handle.
   StrCopy((Char *) MemHandleLock(h), s);
   // unlock the string handle.
   MemHandleUnlock(h);

   // Set the text up
   FldSetTextHandle(field, h);
   if (redraw) FldDrawField(field);
   return errNone;
}

void SetFieldEditable(FieldPtr field, Boolean editable)
{
  FieldAttrType attributes;
  FldGetAttributes(field, &attributes);
  if (editable) {
    attributes.underlined = grayUnderline;
    attributes.editable = 1;
  } else {
    attributes.underlined = noUnderline;
    attributes.editable = 0;
  }
  FldSetAttributes(field, &attributes);
}
